/* *********************************************************************** */ 
/*                                                                         */
/* simcgi.c - CGI app. for WebBench 5.0                                    */
/*                                                                         */
/* Copyright 2002 by Ziff-Davis Media                                      */
/*                                                                         */
/* Authors:                                                                */
/*                                                                         */
/* Ziff-Davis Media                                                        */
/*                                                                         */
/* Abstract:                                                               */
/*                                                                         */
/* Sends Web Server Environment variables to stdout.  Formats in HTML.     */
/* Limits content-length to 1024 characters                                */
/*                                                                         */
/* Revision History:                                                       */
/*                                                                         */
/*       1.0.0 - Original   (Nov 21, 2002)                                 */
/*                                                                         */
/* *********************************************************************** */ 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#define CGI_BUF_SIZE    256
#define SIMCGI_BUF_SIZE 1024

int main(void)
{  char *cgi_vars[] = { "SERVER_SOFTWARE", "SERVER_NAME", "GATEWAY_INTERFACE",
                        "SERVER_PROTOCOL", "SERVER_PORT", "REQUEST_METHOD",
                        "HTTP_ACCEPT", "HTTP_USER_AGENT", "HTTP_REFERER",
                        "PATH_INFO", "PATH_TRANSLATED", "SCRIPT_NAME",
                        "QUERY_STRING", "REMOTE_HOST", "REMOTE_ADDR",
                        "REMOTE_USER", "AUTH_TYPE", "CONTENT_TYPE",
                        "CONTENT_LENGTH", "ANNOTATION_SERVER", "\0" };
   char **cgi_ptr;
   char buffer[SIMCGI_BUF_SIZE+1];
   char tmp_buf[CGI_BUF_SIZE+64];

   void AddToBuffer(char *, int, const char *);
   void PrintBuffer(char *, int, const char *, char);

   /* initialize buffer */
   buffer[0] = (char) NULL;

   /* print header  */
   printf("Content-Type: text/html\n\n");

   /* build start of body  */
   AddToBuffer(buffer, SIMCGI_BUF_SIZE, 
                  "<HTML><HEAD> <TITLE>WebBench 5.0"
                  " CGI Application</TITLE></HEAD><BODY>\r<pre>");

   /* loop through environment variables; printing each one if it exists */
   cgi_ptr = cgi_vars;
   while (**cgi_ptr != '\0') 
   {   char *env_str = getenv(*cgi_ptr);

       /* create new string */
       if ((env_str == NULL) || (strlen(env_str) > CGI_BUF_SIZE))   /* not there */
          sprintf(tmp_buf, "%s = \r", *cgi_ptr);
       else
          sprintf(tmp_buf, "%s = <B>%s</B>\r", *cgi_ptr, env_str);  /* there */

       /* add to buffer  */
       AddToBuffer(buffer, SIMCGI_BUF_SIZE, tmp_buf);

       /* advance pointer */
       cgi_ptr++;
   }

   /* print buffer */
   PrintBuffer(buffer, SIMCGI_BUF_SIZE, "\r</pre></BODY></HTML>\r", ' ');
}


/*
 *
 *  void  AddToBuffer(char *buffer, int max_buf_size, const char *string)
 *
 *  Appends "string" to "buffer".  "buffer" is of size "max_buf_size".
 *  If appending string will overflow buffer then only the first X chars
 *  in string will be added, where X is the number of chars left in buf.
 *  If buffer is already full then AddToBuffer() does nothing.
 *
 */
void AddToBuffer(char *buffer, int max_buf_size, const char *string)
{  int buf_len = strlen(buffer);
   int str_len = strlen(string);

   /* check if buffer is already full  */
   if (buf_len == max_buf_size) return;

   /* check if adding this string will cause buffer overflow  */
   /*  if it does - just add what we can  */
   if ((buf_len + str_len) > max_buf_size)
      strncat(buffer, string, max_buf_size - buf_len);
   else 
      strcat(buffer, string);
}


/*
 *  void  PrintBuffer(char *buffer, int max_buf_size, const char *string, char pad)
 *
 *  Appends "string" to last X characters in buffer, where X is the length
 *  of "string". 
 *
 *  PrintBuffer() will overwrite any characters at end of buffer if needed.
 *  
 *  PrintBuffer() will pad any unused positions in buffer with pad if nec.
 *  
 */
void PrintBuffer(char *buffer, int max_buf_size, const char *string, char pad)
{  int buf_len = strlen(buffer);         /* # of chars in buffer */
   int str_len = strlen(string);         /* size of mand. end-of-text string */
   int offset = max_buf_size - str_len;  /* offset in buffer to place string */
   int i;

   /* error check */  
   if (str_len > max_buf_size) return;   // the string to append is larger than buffer

   /* add padding if nec. - so application will  */
   /*   return same # of bytes on different webserver */
   if ((buf_len + str_len) < max_buf_size)
      for(i=buf_len; i < (max_buf_size - str_len); i++) buffer[i] = pad;

   /* write mandatory end-of-text string at end of buffer */
   for (i=0; i < str_len; i++) 
      buffer[i+offset] = string[i];

   /* output buffer */
   printf("%s", buffer);
}
